/*************************************************************************/
/*************************************************************************/
/*                                                                       */
/*  File Name : EdgeList.h                                               */
/*                                                                       */
/*       Date : May   15, 1999                                           */
/* Updated at : July  10, 1999                                           */
/*                                                                       */
/*     Author : H. Timucin OZDEMIR                                       */
/*                                                                       */
/*************************************************************************/
/**
    EdgeList represents a vehicle duty which can

      1) starts from and ends at the same depot, and
      2) total demand is no more than the maximum capacity of a vehicle.

    The cost is calculated as follows:
      *) Total length of the travel. 
**/
/*************************************************************************/

#ifndef _EDGELIST_H
#define _EDGELIST_H

#include <iostream.h>

#include <gVector.h>  // ../TOOLS
#include "VRPcommons.h" 

class EdgeList;
typedef EdgeList* EdgeListPtr;

#include "Edge.h" 
#include "TimeWindow.h" 
#include "VRProblem.h" 

class EdgeList : protected gVector<Edge> {

  public :

     EdgeList() ;
     EdgeList(int _size) ;
     EdgeList(Edge& edge); 
     EdgeList(const EdgeList& RHS); 

     void addEdge(Edge& edge);

     doesContainNodeId(int nodeId) const;
     void splitListFromNode(int,EdgeList&,EdgeList&,VRProblemPtr);

     EdgePtr getFirstEdge() const; 
     EdgePtr getLastEdge() const ;
     EdgePtr getNthEdge(unsigned indx) const ;

     int   getNumberOfEdges() const ;

     getStartNode() const ;
     getEndNode() const ;

     canFollow(EdgeListPtr RHS, VRProblem* prob) ;

     inline canPrecede(EdgeListPtr RHS, VRProblem* prob) 
     { return (RHS!=NULL?RHS->canFollow(this,prob):1);  };

     void removeEdgesStartFrom(int);
     void removeEdgesEndAt(int);
     void removeFromTo(int,int);

     vrptime getStartDate() const;
     vrptime getEndDate() const;

     operator==(const EdgeList&);
     EdgeList& operator=(const EdgeList&);

     friend ostream& operator<<(ostream& os, const EdgeList& RHS);

     isValid(VRProblem*, int giveReason=0);

     inline void  clear() { myReset(); reset(); };

     canMerge( EdgeListPtr eList, VRProblemPtr problem);
     mergeWith( EdgeListPtr eList );

     getNumberOfDifferences(EdgeListPtr eList);
     TimeWindow getSpan() const;

     inline vrpcost  getDCost() const { return dCost; };

     inline vrpcapacity getDemand() const { return tCapacity; };

     inline vrpcost getCostPerDemand() const 
     { return (getDemand()>0?getDCost()/getDemand():FLT_MAX); }

     // returns total demands between from and to edge
     vrpcapacity getDemandFromTo(int,int,VRProblemPtr prob) const ;

     // returns total demands upto to edge
     inline vrpcapacity getDemandTo(int to, VRProblemPtr prob) const 
     { return getDemandFromTo(0,to,prob); }

     // returns total demands after from edge
     inline vrpcapacity getDemandAfter(int from, VRProblemPtr prob) const 
     { return getDemandFromTo(from,getNumberOfEdges(),prob); }

     static EdgeListPtr combineEdgeLists(EdgeListPtr first,unsigned fIndex,
	          EdgeListPtr second, unsigned sIndex, VRProblemPtr prob);

     static EdgeListPtr combineEdgeLists(EdgeListPtr first, int f0, int f1,
			 EdgeListPtr second,int s0,int s1,VRProblemPtr prob );

     static EdgeListPtr 
     merge2EdgeList(EdgeListPtr first, EdgeListPtr second, VRProblemPtr prob);

     doesCommonEdgeExist(EdgeListPtr eList) const;

     splitEdgeList( vrptime onePX, EdgeListPtr* r0, 
                    EdgeListPtr* r1, VRProblemPtr prob);
     
     splitEdgeList( vrptime onePX, vrptime twoPX, EdgeListPtr* r0, 
                    EdgeListPtr* r1, EdgeListPtr* r2, VRProblemPtr prob);

     doesContainEdge(int from, int to) const;
     void printTabular(ostream&) const;
     inline vrptime getWaitingTime() { return wtime; }

   private:

     void nicePrint(ostream&) const;

     float ecost;
     vrpcost     dCost;            // the distance of the travel 
     vrpcapacity tCapacity;        // the total served customer demands 
     char        dCostCalculated; 
     vrptime     wtime;        

     void myReset();
     void copyFrom(const EdgeList& );

}; // end of Class EdgeList

#endif
